@@ -23,6 +23,7 @@ import (
 
 	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
+	apiclient "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/fields"
@@ -42,14 +43,16 @@ import (
 const (
 	admissionHookSecretName                 = "prometheus-operator-admission"
 	prometheusOperatorServiceDeploymentName = "prometheus-operator"
+	operatorTLSDir                          = "/etc/tls/private"
 )
 
 type Framework struct {
-	KubeClient     kubernetes.Interface
-	MonClientV1    monitoringclient.MonitoringV1Interface
-	HTTPClient     *http.Client
-	MasterHost     string
-	DefaultTimeout time.Duration
+	KubeClient      kubernetes.Interface
+	MonClientV1     monitoringclient.MonitoringV1Interface
+	APIServerClient apiclient.Interface
+	HTTPClient      *http.Client
+	MasterHost      string
+	DefaultTimeout  time.Duration
 }
 
 // New setups a test framework and returns it.
@@ -64,6 +67,11 @@ func New(kubeconfig, opImage string) (*Framework, error) {
 		return nil, errors.Wrap(err, "creating new kube-client failed")
 	}
 
+	apiCli, err := apiclient.NewForConfig(config)
+	if err != nil {
+		return nil, errors.Wrap(err, "creating new kube-client failed")
+	}
+
 	httpc := cli.CoreV1().RESTClient().(*rest.RESTClient).Client
 	if err != nil {
 		return nil, errors.Wrap(err, "creating http-client failed")
@@ -75,11 +83,12 @@ func New(kubeconfig, opImage string) (*Framework, error) {
 	}
 
 	f := &Framework{
-		MasterHost:     config.Host,
-		KubeClient:     cli,
-		MonClientV1:    mClientV1,
-		HTTPClient:     httpc,
-		DefaultTimeout: time.Minute,
+		MasterHost:      config.Host,
+		KubeClient:      cli,
+		MonClientV1:     mClientV1,
+		APIServerClient: apiCli,
+		HTTPClient:      httpc,
+		DefaultTimeout:  time.Minute,
 	}
 
 	return f, nil
@@ -151,9 +160,10 @@ func (f *Framework) MakeEchoDeployment(group string) *appsv1.Deployment {
 // inside the specified namespace using the specified operator image. In addition
 // one can specify the namespaces to watch, which defaults to all namespaces.
 // Returns the CA, which can bs used to access the operator over TLS
-func (f *Framework) CreatePrometheusOperator(ns, opImage string, namespaceAllowlist, namespaceDenylist, prometheusInstanceNamespaces, alertmanagerInstanceNamespaces []string, createRuleAdmissionHooks bool) ([]finalizerFn, error) {
-	tru := true
-	fals := false
+func (f *Framework) CreatePrometheusOperator(ns, opImage string, namespaceAllowlist,
+	namespaceDenylist, prometheusInstanceNamespaces, alertmanagerInstanceNamespaces []string,
+	createRuleAdmissionHooks bool) ([]finalizerFn, error) {
+
 	var finalizers []finalizerFn
 
 	_, err := CreateServiceAccount(
@@ -245,21 +255,18 @@ func (f *Framework) CreatePrometheusOperator(ns, opImage string, namespaceAllowl
 			Name:         "cert",
 			VolumeSource: v1.VolumeSource{Secret: &v1.SecretVolumeSource{SecretName: admissionHookSecretName}}})
 
-	// Use ghostunnel to provide TLS
-	deploy.Spec.Template.Spec.Containers = append(deploy.Spec.Template.Spec.Containers,
-		v1.Container{
-			Name: "ghostunnel",
-			Args: []string{
-				"server",
-				"--listen=:8443",
-				"--target=127.0.0.1:8080",
-				"--key=cert/key",
-				"--cert=cert/cert",
-				"--disable-authentication"},
-			Image:           "squareup/ghostunnel:v1.4.1",
-			Ports:           []v1.ContainerPort{{Name: "https", ContainerPort: 8443}},
-			VolumeMounts:    []v1.VolumeMount{{Name: "cert", MountPath: "/cert", ReadOnly: true}},
-			SecurityContext: &v1.SecurityContext{AllowPrivilegeEscalation: &fals, ReadOnlyRootFilesystem: &tru}})
+	deploy.Spec.Template.Spec.Containers[0].VolumeMounts = append(deploy.Spec.Template.Spec.Containers[0].VolumeMounts,
+		v1.VolumeMount{Name: "cert", MountPath: operatorTLSDir, ReadOnly: true})
+
+	// The addition of rule admission webhooks requires TLS, so enable it and
+	// switch to a more common https port
+	if createRuleAdmissionHooks {
+		deploy.Spec.Template.Spec.Containers[0].Args = append(
+			deploy.Spec.Template.Spec.Containers[0].Args,
+			"--web.enable-tls=true",
+			fmt.Sprintf("--web.listen-address=%v", ":8443"),
+		)
+	}
 
 	err = CreateDeployment(f.KubeClient, ns, deploy)
 	if err != nil {
@@ -314,7 +321,7 @@ func (f *Framework) CreatePrometheusOperator(ns, opImage string, namespaceAllowl
 
 	service.Namespace = ns
 	service.Spec.ClusterIP = ""
-	service.Spec.Ports = append(service.Spec.Ports, v1.ServicePort{Name: "https", Port: 443, TargetPort: intstr.FromInt(8443)})
+	service.Spec.Ports = []v1.ServicePort{{Name: "https", Port: 443, TargetPort: intstr.FromInt(8443)}}
 
 	if _, err := CreateServiceAndWaitUntilReady(f.KubeClient, ns, service); err != nil {
 		return finalizers, errors.Wrap(err, "failed to create prometheus operator service")
